@@ -13,7 +13,7 @@ my $R = Statistics::R->new();
 
 if ( $command eq 'start' ) {
     print ">> R - STARTING... ";
-    if ( $R->start_sharedR ) {
+    if ( $R->start( shared => 1 ) {
         print "OK\n";
         if ( $^O =~ /^(?:.*?win32|dos)$/i ) {
             print "[Press Ctrl+C to terminate].\n";
@@ -32,14 +32,16 @@ if ( $command eq 'start' ) {
     }
 }
 elsif ( $command eq 'stop' ) {
-    $R->stopR();
+    $R->stop();
 }
 
 sub usage {
     my ( $script ) = ( $0 =~ /([^\\\/]+)$/s );
 
     print <<"EOUSAGE";
-Statistics::R - $Statistics::R::VERSION
+Statistics::R version $Statistics::R::VERSION
+
+Start or stop a Perl-R communication bridge
 
 USAGE:
 
